from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.model.level import Level


class LevelRepositoryService(LevelRepositoryABC):

    def __init__(self, logger: DatabaseLogger, db_context: DatabaseContextABC, servers: ServerRepositoryABC):
        self._logger = logger
        self._context = db_context

        self._servers = servers

        LevelRepositoryABC.__init__(self)

    @staticmethod
    def _get_value_from_result(value: any) -> Optional[any]:
        if isinstance(value, str) and 'NULL' in value:
            return None

        return value

    def _level_from_result(self, sql_result: tuple) -> Level:
        return Level(
            self._get_value_from_result(sql_result[1]),  # name
            self._get_value_from_result(sql_result[2]),  # color
            int(self._get_value_from_result(sql_result[3])),  # min xp
            int(self._get_value_from_result(sql_result[4])),  # permissions
            self._servers.get_server_by_id(sql_result[5]),  # server
            id=self._get_value_from_result(sql_result[0])  # id
        )

    def get_levels(self) -> List[Level]:
        levels = List(Level)
        self._logger.trace(__name__, f'Send SQL command: {Level.get_select_all_string()}')
        results = self._context.select(Level.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f'Get level with id {result[0]}')
            levels.append(self._level_from_result(result))

        return levels

    def get_level_by_id(self, id: int) -> Level:
        self._logger.trace(__name__, f'Send SQL command: {Level.get_select_by_id_string(id)}')
        result = self._context.select(Level.get_select_by_id_string(id))[0]

        return self._level_from_result(result)

    def find_level_by_id(self, id: int) -> Optional[Level]:
        self._logger.trace(__name__, f'Send SQL command: {Level.get_select_by_id_string(id)}')
        result = self._context.select(Level.get_select_by_id_string(id))
        if result is None or len(result) == 0:
            return None

        return self._level_from_result(result[0])

    def get_levels_by_server_id(self, server_id: int) -> List[Level]:
        levels = List(Level)
        self._logger.trace(__name__, f'Send SQL command: {Level.get_select_by_server_id_string(server_id)}')
        results = self._context.select(Level.get_select_by_server_id_string(server_id))

        for result in results:
            self._logger.trace(__name__, f'Get level with id {result[0]}')
            levels.append(self._level_from_result(result))

        return levels

    def find_levels_by_server_id(self, server_id: int) -> Optional[List[Level]]:
        levels = List(Level)
        self._logger.trace(__name__, f'Send SQL command: {Level.get_select_by_server_id_string(server_id)}')
        results = self._context.select(Level.get_select_by_server_id_string(server_id))
        if results is None or len(results) == 0:
            return None

        for result in results:
            self._logger.trace(__name__, f'Get level with id {result[0]}')
            levels.append(self._level_from_result(result))

        return levels

    def add_level(self, level: Level):
        self._logger.trace(__name__, f'Send SQL command: {level.insert_string}')
        self._context.cursor.execute(level.insert_string)

    def update_level(self, level: Level):
        self._logger.trace(__name__, f'Send SQL command: {level.udpate_string}')
        self._context.cursor.execute(level.udpate_string)

    def delete_level(self, level: Level):
        self._logger.trace(__name__, f'Send SQL command: {level.delete_string}')
        self._context.cursor.execute(level.delete_string)
